# Schedule - A Zope object to support schedules, a type of calender
#            which does not have overlapping events, and permits events
#            to cross header bounderies (days)
#
# Copyright (C) 2002 Robert A. Klahn (rklahn@acm.org)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

"""Schedule"""
__version__ = "$Id: __init__.py,v 1.5 2002/04/15 05:56:26 rklahn Exp $"	

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Acquisition import Implicit
from Globals import Persistent
from AccessControl.Role import RoleManager
from OFS.SimpleItem import Item
from OFS.PropertyManager import PropertyManager
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from DateTime import DateTime

class ScheduleProduct(Implicit, Persistent, RoleManager, Item,
                      PropertyManager):
    """ Schedule Product Class

        A Schedule is a type of calender for which entries can cross
        day bounderies. Entires are stored in a list of dictionaries.
    """

    meta_type='Schedule'

    security=ClassSecurityInfo()

    manage_options=({'label':'Edit','action':'manage_main'},
                    {'label':'View','action':''},) \
                    + PropertyManager.manage_options \
                    + RoleManager.manage_options + Item.manage_options

    index_html=PageTemplateFile('index_html',globals())

    render_html=PageTemplateFile('render_html',globals())

    overlap_html=PageTemplateFile('overlap_html',globals())

    manage_main=PageTemplateFile('editForm',globals())

    _properties=({'id':'title','type':'string','mode':'w'},
                 {'id':'days','type':'lines','mode':'w'},
                 {'id':'times','type':'lines','mode':'w'}
                 )

    def __init__(self,id,title=None,days=None,times=None):
        self.id = id
        self.title = title
        self.days = days
        self.times = times
        self._scheduleEntries = []
        self._p_changed = 1

    security.declareProtected('Delete Schedule Entry','deleteScheduleItem')
    def deleteScheduleItem(self,startday,endday,starttime,endtime,title,REQUEST=None):
        """ Delete an item from the Schedule
        """
        for Entry in self._scheduleEntries:
            if Entry['title']==title:
                x = Entry['time_start'].split(':')
                entryStartTime=(int(x[0])*60)+int(x[1])
                x = Entry['time_end'].split(':')
                entryEndTime=(int(x[0])*60)+int(x[1])
                x = starttime.split(':')
                callerStartTime=(int(x[0])*60)+int(x[1])
                x = endtime.split(':')
                callerEndTime=(int(x[0])*60)+int(x[1])
                if (entryStartTime <= callerStartTime) and (entryEndTime >= callerEndTime):
                    entryStartDay = self.days.index(Entry['day_start'])
                    entryEndDay = self.days.index(Entry['day_end'])
                    callerStartDay = self.days.index(startday)
                    callerEndDay = self.days.index(endday)
                    if (entryStartDay <= callerStartDay) and (entryEndDay >= callerEndDay):
                        self._scheduleEntries.remove(Entry)
                        self._p_changed = 1
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')

    security.declareProtected('Add Schedule Entry','AddScheduleItem')
    def AddScheduleItem(self,startday,endday,starttime,endtime,title,REQUEST=None):
        """ Delete an item from the Schedule
        """
        newentry = {}
        newentry['day_start']=startday
        newentry['day_end']=endday
        newentry['time_start']=starttime
        newentry['time_end']=endtime
        newentry['title']=title
        # ensure no overlap before adding to array
        callerEndDay=self.days.index(endday)
        callerStartDay=self.days.index(startday)
        x=starttime.split(':')
        callerStartTime=(int(x[0])*60)+int(x[1])
        x=endtime.split(':')
        callerEndTime=(int(x[0])*60)+int(x[1])
        for Entry in self._scheduleEntries:
            entryStartDay=self.days.index(Entry['day_start'])
            entryEndDay= self.days.index(Entry['day_end'])
            x = Entry['time_start'].split(':')
            entryStartTime=(int(x[0])*60)+int(x[1])
            x = Entry['time_end'].split(':')
            entryEndTime=(int(x[0])*60)+int(x[1])
            # Start day/time within existing entry (upper left hand corner of what we are adding)
            if (callerStartDay >= entryStartDay) and (callerStartDay <= entryEndDay):
                if (callerStartTime >= entryStartTime) and (callerStartTime < entryEndTime):
                    if REQUEST is not None:
                        REQUEST['RESPONSE'].redirect(self.absolute_url()+'/overlap_html?message=Start day/time within existing entry')
                        return(1==0)
                    raise "Schedule","Overlapping Entry"
            # End day/time within existing entry (lower right hand corner of what we are adding)
            if (callerEndDay >= entryStartDay) and (callerEndDay <= entryEndDay):
                if (callerEndTime > entryStartTime) and (callerEndTime <= entryEndTime):
                    if REQUEST is not None:
                        REQUEST['RESPONSE'].redirect(self.absolute_url()+'/overlap_html?message=End day/time within existing entry')
                        return(1==0)
                    raise "Schedule","Overlapping Entry"
            # Existing Start day/time within adding entry
            if (entryStartDay >= callerStartDay) and (entryStartDay <= callerEndDay):
                if (entryStartTime >= callerStartTime) and (entryStartTime < callerEndTime):
                    if REQUEST is not None:
                        REQUEST['RESPONSE'].redirect(self.absolute_url()+'/overlap_html?message=Existing Start day/time within adding entry')
                        return(1==0)
                    raise "Schedule","Overlapping Entry"
            # Existing End day/time within adding entry
            if (entryEndDay >= callerStartDay) and (entryEndDay <= callerEndDay):
                if (entryEndTime > callerStartTime) and (entryEndTime <= callerEndTime):
                    if REQUEST is not None:
                        REQUEST['RESPONSE'].redirect(self.absolute_url()+'/overlap_html?message=Existing End day/time within adding entry')
                        return(1==0)
                    raise "Schedule","Overlapping Entry"
        
        self._scheduleEntries.append(newentry)
        self._p_changed = 1
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')
        return(1==1)

    security.declareProtected('View','retscheduleNow')
    def retscheduleNow(self,day):
        """ Return the entry on the schedule (for dayth column) for right
            now. Return an empty string if nothing is on the schedule for
            right now.
        """
        try:
            nowthday = self.days.index(day)
        except:
            raise "Schedule","Day specified not on Schedule"
        nowthmins = (DateTime().hour()*60)+DateTime().minute()
        rettitle = ''
        # Go looking for the entry at this time
        for Entry in self._scheduleEntries:
            entryStartDay=self.days.index(Entry['day_start'])
            entryEndDay= self.days.index(Entry['day_end'])
            x = Entry['time_start'].split(':')
            entryStartTime=(int(x[0])*60)+int(x[1])
            x = Entry['time_end'].split(':')
            entryEndTime=(int(x[0])*60)+int(x[1])
            # Day asked for between entryStartDay and entryEndDay?
            if (nowthday >= entryStartDay) and (nowthday <= entryEndDay):
                # Current time between entryStartTime and entryEndTime?
                # If we are exactly at the begining of one entry, and the
                # end of another, favor the starting one.
                if (nowthmins >= entryStartTime) and (nowthmins < entryEndTime):
                    # Bingo
                    rettitle = Entry['title']
                    break # out of for loop
        return rettitle

    security.declareProtected('View','retscheduleNextChange')
    def retscheduleNextChange(self,day):
        """ Return the number of seconds until the next change on the
            schedule (for the dayth column). This is actualy going
            to return the number of seconds, accurate to within one
            minute.
        """
        try:
            nowthday = self.days.index(day)
        except:
            raise "Schedule","Day specified not on Schedule"
        nowthmins = (DateTime().hour()*60)+DateTime().minute()
        # Used to determine the entry with a start time least greater than right now
        # used if there is no entry at this time to determine the time to the next
        # entry. Initial value of midnight, next day, or, another way to think about it
        # the number of minutes in a day.
        leastGreaterThan = 24*60
        # Go looking for the entry at this time
        for Entry in self._scheduleEntries:
            entryStartDay=self.days.index(Entry['day_start'])
            entryEndDay= self.days.index(Entry['day_end'])
            x = Entry['time_start'].split(':')
            entryStartTime=(int(x[0])*60)+int(x[1])
            x = Entry['time_end'].split(':')
            entryEndTime=(int(x[0])*60)+int(x[1])
            # Day asked for between entryStartDay and entryEndDay?
            if (nowthday >= entryStartDay) and (nowthday <= entryEndDay):
                # Current time between entryStartTime and entryEndTime?
                # If we are exactly at the begining of one entry, and the
                # end of another, favor the starting one.
                if (nowthmins >= entryStartTime) and (nowthmins < entryEndTime):
                    # Bingo
                    deltamins = entryEndTime - nowthmins
                    return str(deltamins*60) # to seconds
                # Not the current entry, but later in the day, and less later than
                # one previously seen
                if (nowthmins < entryStartTime) and (leastGreaterThan < entryStartTime):
                    leastGreaterThan = entryStartTime
        # If we get here, then there is no entry on the schedule for the current time,
        # AND leastGreaterThan is the number of minutes past midnight until the next
        # entry, or if there is no next entry, the number of minutes in a day.
        deltamins = leastGreaterThan - nowthmins
        return str(deltamins*60)

    security.declareProtected('View','retscheduleEntries')
    def retscheduleEntrys(self):
        return self._scheduleEntries

InitializeClass( ScheduleProduct )

manage_addScheduleForm=PageTemplateFile('addForm', globals(),
                                            __name__='manage_addScheduleForm')

def manage_addSchedule(self, id, title='',days=None,times=None, REQUEST=None):
    """ Create a new schedule and add it to myself """
    s=ScheduleProduct(id,title,days,times)
    self._setObject(id,s)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')

def initialize(registrar):
    registrar.registerClass(
        ScheduleProduct, 
        constructors = (manage_addScheduleForm, manage_addSchedule),
        icon='Schedule.png',
        )

    registrar.registerHelp()
    registrar.registerHelpTitle('Zope Help')

